animator={}
--[[ overview

animator.createTableOfQuads(image, qw, qh)
animator.giveAnimationProperties(o,frameTable)
animator.updateAnimation(o,dt)
animator.changeAnimation(o,animation)
animator.drawSprite(o,img,quadTable)
animator.listOfFrameTables

--]]

function animator.createTableOfQuads(image,qw,qh)
	--qw,qh = width and height of quads
	--a reference image

	local quadTable={}
	local lgq=love.graphics.newQuad
	
	local sw=image:getWidth()
	local sh=image:getHeight()
	
	local c=math.floor(sw/qw)
	local r=math.floor(sh/qh)
	
	--rows then columns
	for i=1,r do
		quadTable[i]={}
		for j=1,c do
			quadTable[i][j]=lgq((j-1)*qw,(i-1)*qh,qw,qh,sw,sh)
		end
	end
	
	return quadTable
end


function animator.giveAnimationProperties(o,frameTable)--gives object(o) variables it needs for use with other animation functions

	--these variables should be a function of the sprite sheet.
	--ox,oy:	offset of image from o.x and o.y
	o.ox=13
	o.oy=13
	
	if o.sx ==  nil then o.sx = 1 end
	if o.sy ==  nil then o.sy = 1 end
	
	o.frameTable=frameTable	--frameTable:	starting animation state... Maybe I should rename frameTable to animationStates
	o.frame=1	--frame:	...see below?  I can't remember which is which, I need both in order to have "bounce" style animations.
	o.currentFrame=1	--currentFrame:	current frame of frame Table.  Or current frame of the animation defined by the frame table.
	o.frameTime=0	--frameTime:	counts amount of time spent on current frame

	--these can be a function of what's come before
	local t=o.frameTable[1][3] --may be nil.
	if t==nil then t=0 end

	--maxFrameTime:	the amount of time to stay on current frame  
	o.maxFrameTime=t
	o.maxFrame=#o.frameTable
	
	--[[ REFERENCE: sampe frame table
	run={
		type='bounce',
		{1,4,0.15},
		{1,5,0.15},
		{1,6,0.15}
	}

	--Frame Tables contain 
	--(1) animation type
		--"single", "repeat", or "bounce"
			--"single" runs through once, but stops on last frame
			--"repeat" runs through once, and then starts over
			--"bounce" runs through once, and then runs backwards to beginning, and then starts over
		
	--(2) list of frames in an animation.	
		-- { quad row, quad column, frame duration, mirror}
		--The fourth entry, if there, indicates that the sprite will be flipped horizontally.	
	
	--]]
end


function animator.updateAnimation(o,dt)

	--only update if animation type isn't "single"
	--[[
		Note: "single" has changed.  Now, the animation won't update when BOTH of the following conditions are met:
		1) type='single'
		2) currentFrame==o.maxFrame
	--]]
	
	--[[
	if o.frameTable.type=='single' and o.currentFrame~=o.maxFrame then
		o.frameTime=o.frameTime+dt
		if o.frameTime>o.maxFrameTime then
			o.frame=o.frame+1
			o.currentFrame=o.frame
			o.frameTime=0
		end			
		
	end
	
	if o.frameTable.type~='single'--]]

	if o.frameTable.type=='single' and o.currentFrame==o.maxFrame then --do nothing
	
	else	

		o.frameTime=o.frameTime+dt
		
		--only change current frame's duration is over
		if o.frameTime>o.maxFrameTime then
			o.frame=o.frame+1
			o.currentFrame=o.frame
			o.frameTime=0
			
			--if frame is beyond the frameTable
			if o.frame>o.maxFrame then
							
				if o.frameTable.type=='repeat' then
					o.frame=1
					o.currentFrame=1
					
				elseif o.frameTable.type=='bounce' then
				
					o.currentFrame=2*o.maxFrame-o.frame
					if o.currentFrame==1 then
				--test love.graphics.setBackgroundColor(0,0,100)
						o.frame=1
						o.currentFrame=1
					end
				end
				
			end
			
			o.maxFrameTime=o.frameTable[o.currentFrame][3]
		end
		
	end
	
end



--o is an object with certain variables (see notes in function)
--img is a 'drawable'
--quadTable is a table with quads.  The precise quad to look up is a property of the object(o)

function animator.drawSprite(o,img,quadTable)

	--2/15/2015
	if img==nil then img = o.spriteSheet end
	if quadTable==nil then quadTable = o.quadTable end


	--[[ o is an object with
		o.frameTable
		o.x
		o.y
		o.w
		o.h
		
		
		o.currentFrame
		
		maybe later will also need
		o.r
		o.sx (but probably not o.sy)
	--]]
	
	--[[ img is a single sprite sheet.
		if using a spriteSheetTable, this input will have to be spriteSheetTable[i], where i is an integer.
		
		The reason to only have a single image as an input is in order to be able to specify the color of each layer of the image.
	--]]
	
	--[[ quadTable is what you'd get from animation.getQuadTable()
	
	--]]
	
	
	local a,b=1,0 --adjustment
	
	
	--note that the way frameTables define quads are now rows x columns, i.e. y,x, rather than x,y.
	local qy=o.frameTable[o.currentFrame][1]
	local qx=o.frameTable[o.currentFrame][2]
	
	if o.frameTable[o.currentFrame][4] then
		--sx=sx*-1
		--mirror image
		a=-1
		b=o.w/math.abs(o.sx)
	end
	
	local turn_adjustment = 0
	if o.sx<0 then turn_adjustment = o.w end
	
	---[[BLUE
	--love.graphics.setColor(0,255,255)
	love.graphics.push()
		love.graphics.translate(o.w/2,o.h/2)
		love.graphics.draw(
			img,
			quadTable[qy][qx],
			o.x,o.y,
			o.r, --r?
			o.sx*a, --sx?
			o.sy, --sy?
			o.ox+o.w/o.sx/2-turn_adjustment/o.sx,--ox
			o.oy+o.h/o.sy/2--oy
			)
	love.graphics.pop()
	--]]
		
	love.graphics.setColor(255,255,255)
	

end



--[[--------------------------------------------------------------
--_________________________________________________________________
The following is linked to my classic mega man srite sheet.
Once I start having multiple sprite sheets, I will need to use different 
versins of these functions.
	changeAnimation(o,animation)
	listOfFrameTables = {}
--]]--=================================================================


function animator.changeAnimation(o,animation)

	local a = animator.listOfFrameTables
	if animation==nil then --nothing yet
	
	elseif animation == "pose1" 				then a = animator.listOfFrameTables.pose1
	elseif animation == "pose2" 				then a = animator.listOfFrameTables.pose2
	elseif animation == "pose3" 				then a = animator.listOfFrameTables.pose3
	elseif animation == "pose4" 				then a = animator.listOfFrameTables.pose4
	
	elseif animation == "stand" 				then a = animator.listOfFrameTables.stand
	elseif animation == "run" 				then a = animator.listOfFrameTables.run
	elseif animation == "jump" 				then a = animator.listOfFrameTables.jump
	elseif animation == "dash" 				then a = animator.listOfFrameTables.dash
	elseif animation == "wallslide" 			then a = animator.listOfFrameTables.wallslide
	elseif animation == "climb" 				then a = animator.listOfFrameTables.climb
	elseif animation == "recoil" 			then a = animator.listOfFrameTables.recoil
	elseif animation == "recoil2" 			then a = animator.listOfFrameTables.recoil2
		
	elseif animation == "stand_shoot" 		then a = animator.listOfFrameTables.stand_shoot
	elseif animation == "run_shoot" 			then a = animator.listOfFrameTables.run_shoot
	elseif animation == "jump_shoot" 		then a = animator.listOfFrameTables.jump_shoot
	elseif animation == "dash_shoot" 		then a = animator.listOfFrameTables.dash_shoot
	elseif animation == "wallslide_shoot" 	then a = animator.listOfFrameTables.wallslide_shoot
	elseif animation == "climb_shoot" 		then a = animator.listOfFrameTables.climb_shoot

	elseif animation == "stand_shoot2" 		then a = animator.listOfFrameTables.stand_shoot2
	elseif animation == "run_shoot2" 		then a = animator.listOfFrameTables.run_shoot2
	elseif animation == "jump_shoot2" 		then a = animator.listOfFrameTables.jump_shoot2
	elseif animation == "dash_shoot2" 		then a = animator.listOfFrameTables.dash_shoot2
	elseif animation == "wallslide_shoot2" 	then a = animator.listOfFrameTables.wallslide_shoot2
	elseif animation == "climb_shoot2" 		then a = animator.listOfFrameTables.climb_shoot2

	elseif animation == "stand_carry" 		then a = animator.listOfFrameTables.stand_carry
	elseif animation == "run_carry" 			then a = animator.listOfFrameTables.run_carry
	elseif animation == "jump_carry" 		then a = animator.listOfFrameTables.jump_carry
	elseif animation == "dash_carry" 		then a = animator.listOfFrameTables.dash_carry
	elseif animation == "wallslide_carry" 	then a = animator.listOfFrameTables.wallslide_carry
	elseif animation == "stand_carryHeavy" 	then a = animator.listOfFrameTables.stand_carryHeavy

	elseif animation == "stand_throw" 		then a = animator.listOfFrameTables.stand_throw
	elseif animation == "run_throw" 			then a = animator.listOfFrameTables.run_throw
	elseif animation == "jump_throw" 		then a = animator.listOfFrameTables.jump_throw
	elseif animation == "dash_throw" 		then a = animator.listOfFrameTables.dash_throw
	elseif animation == "wallslide_throw" 	then a = animator.listOfFrameTables.wallslide_throw
	elseif animation == "climb_throw" 		then a = animator.listOfFrameTables.climb_throw

	elseif animation == "talk" 				then a = animator.listOfFrameTables.talk
	elseif animation == "jump_spin" 			then a = animator.listOfFrameTables.jump_spin
	elseif animation == "jump_spin2" 		then a = animator.listOfFrameTables.jump_spin2
	elseif animation == "slide" 				then a = animator.listOfFrameTables.slide
	elseif animation == "slide_kick" 		then a = animator.listOfFrameTables.slide_kick

	elseif animation == "fall" 				then a = animator.listOfFrameTables.fall
	elseif animation == "sleep" 				then a = animator.listOfFrameTables.sleep
	elseif animation == "body_slam" 			then a = animator.listOfFrameTables.body_slam
	elseif animation == "spin" 				then a = animator.listOfFrameTables.spin
	elseif animation == "look_up" 			then a = animator.listOfFrameTables.look_up
	elseif animation == "roll" 				then a = animator.listOfFrameTables.roll
	elseif animation == "kneel" 				then a = animator.listOfFrameTables.kneel
	elseif animation == "kneel_shoot" 		then a = animator.listOfFrameTables.kneel_shoot
	elseif animation == "kneel_carry" 		then a = animator.listOfFrameTables.kneel_carry
	elseif animation == "kneel_throw" 		then a = animator.listOfFrameTables.kneel_throw
	end

	local frameTable=a
	o.frameTable=a
	
	o.maxFrame=#frameTable
	o.animation=animation
	
	if o.currentFrame>o.maxFrame then
		o.currentFrame=1
		o.frame=1
	end
	
	if o.frameTable.type=='single' then
		o.currentFrame=1
		o.frame=1
		o.frameTime=0
	end

	o.maxFrameTime=frameTable[o.currentFrame][3]
	--[[ note:
		Often this function should be followed by
		currentFrame=1.
		Exceptions include running and shooting, during which you don't want to reset the currentFrame.
	--]]
	
	if animation=="run" or animation == "run_shoot" then
	else o.currentFrame=1
	end
end




--july 2013, so this is a table which documents the various animations that go with the classic Mega Man sprite sheet.

--[[
-- I think I'm in need of GitHub and a wiki
--I think I need to move the frame tables to something like mmClassicAnimations.lua

--contains frameTables
	--stand
	--run
	--jump
	--dash
	--wallslide
	--climb
	--recoil
	--stand_shoot
	--run_shoot
	--jump_shoot
	--dash_shoot
	--wallslide_shoot
	--climb_shoot

	--stand_shoot2
	--run_shoot2
	--jump_shoot2
	--dash_shoot2
	--wallslide_shoot2
	--climb_shoot2

	--stand_carry
	--run_carry
	--jump_carry
	--dash_carry
	--wallslide_carry
	--stand_carryHeavy
	
	--stand_throw
	--run_throw
	--jump_throw
	--dash_throw
	--wallslide_throw
	--climb_throw
	
	--talk
	--jump_spin
	--jump_spin2
	--slide
	--slide_kick

	--fall
	--sleep
	--body_slam
	--spin
	--look_up
	--roll
	--kneel
	--kneel_shoot
	--kneel_carry
	--kneel_throw

	
--Frame Tables contain 
	--(1) animation type
		--"single", "repeat", or "bounce"
		--"single" runs through once, but stops on last frame
		--"repeat" runs through once, and then starts over
		--"bounce" runs through once, and then runs backwards to beginning, and then starts over
		
	--(2) list of frames in an animation.	
		-- { quad row, quad column, frame duration, mirror}
		--The fourth entry, if there, indicates that the sprite will be flipped horizontally.	

--Functions
	--animate.getProperties(o,frameTable)
	--animate.getSpriteSheetTable(imgDir)
	--animate.getQuadTable(image,qw,qh)
	--animate.drawFrame(o,img,quadTable)
	--animate.updateFrame(o,dt)
	--animate.chagneFrameTable(o,frameTable)

--animate.getProperties(o,frameTable)
	--
--animate.getSpriteSheetTable(imgDir)
	--
--animate.getQuadTable(image,qw,qh)
	--
--animate.drawFrame(o,img,quadTable)
	--
--animate.updateFrame(o,dt)
	--
--animate.chagneFrameTable(o,frameTable) -- TYPO "chagne"
	--
--]]	
animator.listOfFrameTables={

	--what used to be listOfFrameTables.  It really SHOULD be different, but for my purposes I just want to limit what I have lying around.  Maybe for a later project I'll make a separate litsOfFrameTables, when I have alternate spriteSheet sets, which have different frames and frame properties.

	--[[1. animation type: single, repeat, bounce
		
		"single" runs through once, but stops on last frame
		
		"repeat" runs through once, and then starts over
		
		"bounce" runs through once, and then runs backwards to beginning, and then starts over
	--]]

	--[[2. list of frames in animation
	
		{ quad row, quad column, frame duration, mirror}
	
		The fourth entry, if there, indicates that the sprite will be flipped horizontally.
	
	--]]

	
	-- 7/8/2015 enemy spriteSheet
	pose1	=	{type = 'single', {6,2,0.3},	{6,1}},
	pose2	=	{type = 'single', {6,3}},
	pose3	=	{type = 'single', {7,6}},
	pose4	=	{type = 'single', {4,2}},
	
	
	--BASIC SPRITES
	stand={
		type='repeat', 
		{1,1,1.0},
		{1,2,0.1},
		--{1,1,0.2},
		--{1,3,0.1}		
	},
	
	--7/4/2015. 
	startRun={type='single',{1,3}}, -- The time this state persists should depend on the run speed.  The rules for this are too complicated to deal with in the animation table.  They should be considered when the player changes states.
	
	run={
		type='bounce',
		{1,4,0.15},
		{1,5,0.15},
		{1,6,0.15}
	},
	jump={
		type='single',
		{1,7}
	},
	dash={
		type='single',
		{1,8}
	},
	wallslide={
		type='single',
		{1,9}
	},
	climb={
		type='repeat',
		{1,10,0.2},
		{1,10,0.2,true} --flip horizontally
	},
	recoil={
		type='single', -- * see change discussed above
		{2,1,.3},
		{2,2}
	},	
	recoil2={
		type='single', -- * see change discussed above
		{2,1}
	},
	
	--END OF BASIC SPRITES
	--ADD SHOOTING
	
	stand_shoot={
		type='single', 
		{2,3,0.1}
	},
	run_shoot={
		type='bounce',
		{2,4,0.15},
		{2,5,0.15},
		{2,6,0.15}
	},
	jump_shoot={
		type='single',
		{2,7}
	},
	dash_shoot={
		type='single',
		{2,8}
	},
	wallslide_shoot={
		type='single',
		{2,9}
	},
	climb_shoot={
		type='single',
		{2,10,0.1}
	},

	--END SHOOT

	--test: adding recoil to shoot
	
	stand_shoot2={
		type='single', 
		{2,3,0.1},
		{8,3}
	},
	run_shoot2={ -- this one is the same as the normal shoot
		type='bounce',
		{2,4,0.15},
		{2,5,0.15},
		{2,6,0.15}
	},
	jump_shoot2={
		type='single',
		{2,7,0.1},
		{8,7}
	},
	dash_shoot2={
		type='single',
		{2,8,0.1},
		{8,8}
	},
	wallslide_shoot2={
		type='single',
		{2,9,0.1},
		{8,9}
	},
	climb_shoot2={
		type='single',
		{2,10,0.1},
		{8,10}
	},

	--END SHOOT	
	
	--START CARRY

	stand_carry={
		type='repeat', 
		{3,1,0.1},
		{3,2,0.1},
		{3,1,0.1},
		{3,3,0.1}		
	},
	run_carry={
		type='bounce',
		{3,4,0.15},
		{3,5,0.15},
		{3,6,0.15}
	},
	jump_carry={
		type='single',
		{3,7}
	},
	dash_carry={
		type='single',
		{3,8}
	},
	wallslide_carry={
		type='single',
		{3,9}
	},
	stand_carryHeavy={
		type='repeat',
		{4,1,.1},
		{4,2,.5}
	},
	
	--END CARRY
	--START THROW
	stand_throw={
		type='single', 
		{4,3,0.1}
	},
	run_throw={
		type='bounce',
		{4,4,0.15},
		{4,5,0.15},
		{4,6,0.15}
	},
	jump_throw={
		type='single',
		{4,7}
	},
	dash_throw={
		type='single',
		{4,8}
	},
	wallslide_throw={
		type='single',
		{4,9}
	},
	climb_throw={
		type='single',
		{4,10,0.1}
	},	

	--END THROW
	--SPECIAL ACTIONS
	talk={
		type='repeat',
		{6,3,0.1},
		{1,1,0.1}
	},
	
	jump_spin={
		type='single',
		{5,7,0.1},
		{5,8,0.1},
		{5,7,0.1,true}, --mirror image
		{1,7}
	},

	jump_spin2={
		type='repeat',
		{5,7,0.1},
		{5,8,0.1},
		{5,7,0.1,true}, --mirror image
		{1,7,0.1}
	},
	
	slide={
		type='single',
		{5,3}
	},
	slide_kick={
		type='single',
		--{1,8,0.1},
		{5,4,0.1},
		{5,5,0.1},
		{5,6,0.1},
		{5,3,0.5}
	},
	fall={
		type='single',
		{6,4,0.2},
		{6,5,0.2},
		{6,6,0.2}
	},
	sleep={
		type='single',
		{6,7}
	},
	body_slam={
		type='single',
		{6,8,0.2},
		{6,9,0.2},
		{6,10,0.2},
		{6,9,0.2},
		{6,8,0.2}
	},
	spin={
		type='repeat',
		{1,1,0.1},
		{7,6,0.1},
		{7,7,0.1},
		{7,8,0.1},
		{7,9,0.1},
		{7,10,0.1},
		{7,9,0.1,true},
		{7,8,0.1,true},
		{7,7,0.1,true},
		{7,6,0.1,true},
		{1,1,0.1,true}
	},
	look_up={
		type='single',
		{7,1}
	},
	roll={
		type='single',
		{5,9}
	},	
	--TESTS
	test={
		type='single',
		{7,11}
	},		
	
	test_standing={
		type='repeat',
		{1,1,0.2},
		{1,2,0.2},
		{1,3,0.2},
		{3,1,0.2},
		{3,2,0.2},
		{3,3,0.2},
		{4,1,0.2},
		{4,2,0.2},
		{4,3,0.2},
		--[[
		{5,1,0.2},
		{5,2,0.2},
		{6,1,0.2},
		{6,2,0.2},
		{6,3,0.2},
		{7,1,0.2},
		{7,2,0.2},
		{7,3,0.2},
		{7,4,0.2},
		{7,5,0.2},
		{7,9,0.2},
		{7,10,0.2},
		--]]
	},
	test_dashing={
		type='repeat',
		{1,8,0.2},
		{2,8,0.2},
		{3,8,0.2},
		{4,8,0.2},
	},
	test_running={
		type='repeat',
		{1,4,0.2},
		{2,4,0.2},
		{3,4,0.2},
		{4,4,0.2},
		{1,5,0.2},
		{2,5,0.2},
		{3,5,0.2},
		{4,5,0.2},
		{1,6,0.2},
		{2,6,0.2},
		{3,6,0.2},
		{4,6,0.2}
	},
	test_jumping={
		type='repeat',
		{1,7,0.2},
		{2,7,0.2},
		{3,7,0.2},
		{4,7,0.2}
	},
	test_wallsliding={
		type='repeat',
		{1,9,0.2},
		{2,9,0.2},
		{3,9,0.2},
		{4,9,0.2}
	},
	
	--KNEELING
	kneel={
		type='single',
		{1,11}
	},
	kneel_shoot={
		type='single',
		{2,11}
	},
	kneel_carry={
		type='single',
		{3,11}
	},
	kneel_throw={
		type='single',
		{4,11}
	}	
}


