
controller={{}}
-- be able to support multiple controllers, and use them like
-- 		controller[1].key[1] = {name = 'up', bind = 'up', isDown = false}

--=====================================
-- Create functions to create controllers.  This will allow me to create and maintain controller objects.
--=====================================
-- EXAMPLES
-- controller[1] = controller.newController()
-- controller.updateKeys(controller[1], controller[1].defaultController[1])
-- controller[1].menu = controller.newMenu()

function controller.newController(keys, x,y,dh,w) --  I think I need to use this function to create all the controllers, even the default controllers, since I know not how to use metatables in lua.
	local c ={
			--{	
			key = {}, default = {}, custom = {}, menu = nil,-- {x,y} -- The menu object exists to contain unique coordinates, in the even that I have multiple controllers and want there to be multiple menus on screen at the same time.  They cannot all be in the same place. This variable should not to be confused with the key shortcut in the row below this. 
			up, down, left, right, act1, act2, move1, move2, menu1, menu2, extra1, extra2, -- These are all shortcuts that correspond to entries in the key table.  They exist to make writing the code a bit more convenient.
			--},
		}
		
	c.key={
		{name = 'up',	bind = 'up', 		isDown = false},
		{name = 'down',	bind = 'down', 		isDown = false},
		{name = 'left',	bind = 'left', 		isDown = false},
		{name = 'right',	bind = 'right', isDown = false},
		{name = 'act 1',	bind = 'd', 	isDown = false},
		{name = 'act 2',	bind = 'space', isDown = false},
		{name = 'move 1',	bind = 's', 	isDown = false},
		{name = 'move 2',	bind = 'a', 	isDown = false},
		{name = 'menu 1',	bind = 'return',isDown = false},
		{name = 'menu 2',	bind = 'w', 	isDown = false},
		{name = 'extra 1',	bind = 'e', 	isDown = false},
		{name = 'extra 2',	bind = 'f', 	isDown = false},
	}
	
	c.default[1] = {'up','down','left','right','d','space','s','a','return','w','e','f'}
	c.default[2] = {'w','s','a','d','right','left','space','lshift','up','down','q','e'}
	c.default[3] = {'up','down','left','right','kp0','kp0','up','down','return','rshift','rctrl','/'}
	
	c.custom = {{},{},{}}

	--[[
	c.up = 		c.key[1]
	c.down = 	c.key[2]
	c.left = 	c.key[3]
	c.right = 	c.key[4]
	c.act1 = 	c.key[5]
	c.act2 = 	c.key[6]
	c.move1 = 	c.key[7]
	c.move2 = 	c.key[8]
	c.menu1 = 	c.key[9]
	c.menu2 = 	c.key[10]
	c.extra1 = 	c.key[11]
	c.extra2 = 	c.key[12]
	--]]
	
	controller.updateKeys(c, keys)
	
	c.menu = controller.newMenu(x,y,dh,w)
	
	
	
	return c
end

function controller.updateKeys(c,newKeys)
	
	if newKeys then
		for i = 1, #newKeys do
			if newKeys[i]==nil then 
				-- skip.
			elseif c.key[i] then -- make sure key exists before changing it.
				c.key[i].bind = newKeys[i] 
			end
		end
	end

	c.up = 		c.key[1]
	c.down = 	c.key[2]
	c.left = 	c.key[3]
	c.right = 	c.key[4]
	c.act1 = 	c.key[5]
	c.act2 = 	c.key[6]
	c.move1 = 	c.key[7]
	c.move2 = 	c.key[8]
	c.menu1 = 	c.key[9]
	c.menu2 = 	c.key[10]
	c.extra1 = 	c.key[11]
	c.extra2 = 	c.key[12]
	
end

function controller.newMenu(x,y,dh,w)
	local menu = controller.menu
	local m = { 
		edit = menu.edit,--false,
		x = menu.x,--1070, 		
		y = menu.y,--50, 
		fontSize = menu.fontSize,--25, 
		w = menu.w,--200,
		}
	
	if x then m.x = x end
	if y then m.y = y end
	if dh then m.fontSize = dh end
	if w then m.w = w end


	--m.font = love.graphics.newFont(m.fontSize)

	return m
end


--===============================
-- 3. Create the Controller Menu for its functions. Each controller[i] willl also its own menu objects for variables. 
--===============================

controller.menu = {}

controller.menu = { 
	edit = false,
	x = 1070, 		-- These x and y values may be unique to specrific controllers.
	y = 50, 
	fontSize = 25, 
	w = 200,
	}

--===================================================================
-- 3.1. Determine button locations and titles on the Controller Menu.
--===================================================================

function controller.menu.load(c)--, x,y,w,dh)
	-- c could be controller[i]
	
	-- GET INPUTS
	if c == nil then c = controller[1] end -- use controller 1 by default.

	c.menu.h = c.menu.fontSize * ( #c.key + 8 ) + 10
	c.menu.font = love.graphics.newFont(c.menu.fontSize - 7 )	
	
	local dy = c.menu.fontSize
	local dx = dy
	local x = c.menu.x + 10
	local y = c.menu.y
	local w = c.menu.w -10
		
	local numberOfCustomOptions = #c.custom--3--#controller.menu.custom -- THIS HAS BEEN REPLACED
	local numberOfDefaultOptions = #c.default--3 -- number of default options.
	
	local k = #c.key
	c.menu.button={
			keyBind = {title = "PLAYER CONTROL", x = x, y = y + dy},	
			default = {title = "LOAD PRESET", x = x, y = y + dy*( k + 2 )},
			saveCustom={title = "SAVE CUSTOM", x = x, y = y + dy*( k + 4 )}, 
			loadCustom={title="LOAD CUSTOM", x = x, y = y + dy*( k + 6 )}
		}

	local b = c.menu.button

	for i = 1, k do	-- coordinates of keybind buttons
		b.keyBind[i] = {
			title = ' '..i..'. "'..c.key[i].bind..'" ('..c.key[i].name..')', -- Note: It's possible that I don't actually use this in the draw function, in order to avoid having to update this variable.
			x =x,
			y = b.keyBind.y + dy*i,
			w = w,
			h = dy,
			}
	end

	for i = 1, numberOfDefaultOptions do	-- coordinates of restore default buttons
		b.default[i] = {
			title = i,
			x = x + dx*(i-1),
			y = b.default.y+dy,
			w = dx, h = dy
		}
	end

	for i = 1, numberOfCustomOptions do
		local y = b.saveCustom.y
		local x = b.saveCustom.x+(i-1)*dx --+ 100
		b.saveCustom[i] = {title = i, y = y+dy, 	x = x, w = dx, h = dy}
		b.loadCustom[i] = {title = i, y = y+dy*3, x = x, w = dx, h = dy}
	end

end
--=============================================
--3.2. DRAWING THE MENU.
--=============================================

function controller.menu.writeTitle(c,b)
	
	if c == nil then c = controller[1] end
	
	local m = c.menu
	if m == nil then m = controller.menu end
	
	love.graphics.setFont(c.menu.font)
	love.graphics.print(b.title,b.x,b.y)
	love.graphics.print(b.title,b.x+1,b.y)
end


function controller.menu.draw(c, title)

	if c == nil then c = controller[1] end
	
	local m = c.menu
	if m == nil then m = controler.menu end
	
	local lgp = love.graphics.printf
	local lg = love.graphics
	local mx,my = love.mouse.getPosition()
	
	love.graphics.setFont(m.font)
	
	---[[background
	lg.setColor(30,30,30)
	if mx > m.x and mx < m.x + m.w and my > m.y and my < m.y + m.h then 
		--lg.setColor(0,0,50)
	end
	lg.rectangle('fill',m.x,m.y,m.w,m.h)
	--]]
	
	--[[OUTLINE, just to test.
	if mx > m.x and mx < m.x + m.w and my > m.y and my < m.y + m.h then 
		lg.setColor(0,255,255)
		lg.rectangle('line',m.x,m.y,m.w,m.h) -- outline menu area
	end--]]	
	
	--INSTRUCTIONS
	local instructions = "mouse over to edit"
	local align = 'left'
	love.graphics.setColor(0,255,0)
	
	if mx > m.x and mx < m.x + m.w and my > m.y and my < m.y + m.h then --
		if m.edit == false then
			instructions = 'click to edit'
			love.graphics.setColor(0,255,255)
		
		elseif m.edit == true then
			instructions = 'press key to edit'
			love.graphics.setColor(255,255,0)
			--align = 'center'
		end
	end
	
	lgp(instructions,m.x,m.y,m.w,align)
	
	-- TURN EDIT MODE OFF when the mouse is outside the menu area for the keybinds
	if mx > m.x and mx < m.x + m.w and my > m.y and my < m.button.default.y then
	else
		m.edit = false
	end
	
	--PLAYER CONTROLS
	local k = m.button.keyBind
	love.graphics.setColor(255,255,255)
	if m.edit and my < m.button.default.y then 
		lg.setColor(255,100,0)
	end
	--m.writeTitle(k)
	if title then
		love.graphics.setFont(c.menu.font)
		love.graphics.print(title,k.x,k.y)
		love.graphics.print(title,k.x+1,k.y)		
	else
		controller.menu.writeTitle(c,k)
	end
	
	love.graphics.setColor(255,255,255)
	for i, v in ipairs(k) do

		lg.setColor(255,255,255)
		if c.key[i].isDown then
			--lg.rectangle('fill',v.x,v.y,v.w,v.h)
		end	
		
		if m.edit and mx > v.x and mx < v.x + v.w and my > v.y and my < v.y + v.h then
			lg.rectangle('line',v.x,v.y,v.w,v.h)
			--[[
			if c.key[i].isDown then
				lg.rectangle('fill',v.x,v.y,v.w,v.h)
			end--]]
		end
		
		lg.setColor(255,255,255)
		if c.key[i].isDown then
			lg.setColor(255,200,0)
		end	

		lgp(' '..i..'.', v.x, v.y, v.w)
		lgp(c.key[i].name, v.x+30, v.y, v.w)
		
		local showTemp = false
		
		for ii,d in ipairs(m.button.default) do -- for each default button, check if mouse is over
			if mx > d.x and mx < d.x + d.w and my > d.y and my < d.y + d.h then
				--if mouse is over, then display the default controls in the keybind column
				if love.mouse.isDown(1) then lg.setColor (0,255,0) else lg.setColor(0,200,0) end
				for iii = 1,#k do
					local k = m.button.keyBind[iii]
					lgp('"'..c.default[ii][iii]..'"', k.x, k.y, k.w-10,'right')			
				end
				showTemp = true
			end
		end
		
		for ii,custom in ipairs(m.button.loadCustom) do -- for each load custom button, check if mouse is over
			if #c.custom[ii]>0 and 
				mx > custom.x and mx < custom.x +  custom.w and my >  custom.y and my < custom.y + custom.h then
				--if mouse is over, then display the default controls in the keybind column
				if love.mouse.isDown(1) then lg.setColor (0,255,0) else lg.setColor(0,200,0) end
				for iii = 1,#k do
					local k = m.button.keyBind[iii]
					lgp('"'..c.custom[ii][iii]..'"', k.x, k.y, k.w-10,'right')			
				end
				showTemp = true
			end
		end			
	
		if showTemp == false then
			lgp('"'..c.key[i].bind..'"', v.x, v.y, v.w-10,'right')			
		end
	end
		
	-- RESTORE DEFAULT
	local d = m.button.default	-- THIS MAY HAVE BEEN REPLACED
	lg.setColor(255,255,255)
	if --controller.menu.edit and 
		mx > m.x and mx < m.x + m.w and
		my > m.button.default.y and
		my < m.button.saveCustom.y	then 
			lg.setColor(255,100,0)
	end		
	controller.menu.writeTitle(c,d)
	
	for i, v in ipairs(d) do
		
		if mx > v.x and mx < v.x + v.w and my > v.y and my < v.y + v.h then
			lg.setColor(0,100,0)
			lg.rectangle('fill',v.x,v.y,v.w,v.h)
			--lgp(v.title, v.x, v.y, v.w, 'center')
		else
		
		end
			lg.setColor(255,255,255)
			lgp(v.title, v.x, v.y, v.w, 'center')
			lg.rectangle('line',v.x,v.y,v.w,v.h)
	end
	
	--SAVE CUSTOM
	if --controller.menu.edit and 
		mx > m.x and mx < m.x + m.w and
		my > m.button.saveCustom.y and
		my < m.button.loadCustom.y	then 
			lg.setColor(255,100,0)
	end		
	
	controller.menu.writeTitle(c, m.button.saveCustom)
	for i, v in ipairs(m.button.saveCustom) do
		if mx > v.x and mx < v.x + v.w and my > v.y and my < v.y + v.h then
			--if #c.custom[i]>0 then 
				lg.setColor(0,100,0)
			--else
			--	lg.setColor(0,0,0)
			--end
			lg.rectangle('fill',v.x,v.y,v.w,v.h)
			--lgp(v.title, v.x, v.y, v.w, 'center')
		else
		
		end
			lg.setColor(255,255,255)
			lgp(v.title, v.x, v.y, v.w, 'center')
			lg.rectangle('line',v.x,v.y,v.w,v.h)
	end
	
	-- LOAD CUSTOM
	if --controller.menu.edit and
		mx > m.x and mx < m.x + m.w and
		my > m.button.loadCustom.y and
		my < m.y + m.h	then 
			lg.setColor(255,100,0)
	end				
	controller.menu.writeTitle(c, m.button.loadCustom)
	for i, v in ipairs(m.button.loadCustom) do
		if mx > v.x and mx < v.x + v.w and my > v.y and my < v.y + v.h then
			if #c.custom[i]>0 then 
				lg.setColor(0,100,0)
			else
				lg.setColor(0,0,0)
			end
			lg.rectangle('fill',v.x,v.y,v.w,v.h)
			--lgp(v.title, v.x, v.y, v.w, 'center')
		else
		
		end
			lg.setColor(255,255,255)
			lgp(v.title, v.x, v.y, v.w, 'center')
			lg.rectangle('line',v.x,v.y,v.w,v.h)
	end
	
end

function controller.draw(c) -- I don't think I even need this?  Or if I do, it should be something other than the menu draw function.
	--controller.menu.draw(controller[1]) -- controller.menu.draw(c)
	
	if c == nil then
		for i = 1, #controller do
			local c = controller[i]
			if c.menu then controller.menu.draw(c) end
		end
	else
		if c.menu then controller.menu.draw(c) end
	end
	
end

function controller.keypressed(c, key, scancode, isrepeat)
	
	--for i = 1, #controller do
	--	local c = controller[i]
		for j = 1, #c.key do
			if key == c.key[j].bind then c.key[j].isDown = true end
		end
	--end
	
	

end

function controller.menu.keypressed(c, key)

	--for i = 1, #controller do
		--local c = controller[i]
		if c.menu then
			if c.menu.edit then
				local mx,my = love.mouse.getPosition()
				for i,v in ipairs(c.menu.button.keyBind) do
					if mx > v.x and mx < v.x + v.w and my > v.y and my < v.y + v.h then
						c.key[i].bind = key
						controller.updateKeys(c) -- am I using this function correctly?
					end
				end
			end
		end
	--end	

end

function controller.keyreleased(c, key, scancode, isrepeat)
	
	--for i = 1, #controller do
		--local c = controller[i]
		for j = 1, #c.key do
			if key == c.key[j].bind then c.key[j].isDown = false end
		end
	--end
	
end

function controller.menu.mousepressed(c, mx, my, button, istouch)
	
	--for i = 1, #controller do  
		local m = c.menu	-- will no longer work.  Now needs to be controller[i].menu
		if m then
			if mx > m.x and mx < m.x + m.w and my > m.y and my < m.button.default.y then
				if m.edit == false then m.edit = true
				elseif m.edit == true then m.edit =  false 
				end
			end
		--=======================================


			local b = m.button --controller.menu.button	-- may no longer work.  Check if it's not controller[i].menu.button.
			--local c = controller[i]--.custom--controller.menu.custom	-- may no longer work.  Check if it's not now controller[i].custom.
			local k = c.key --controller.key
			local d = c.default
			

			for i, v in ipairs(b.default) do
				if mx > v.x and mx < v.x + v.w and my > v.y and my < v.y + v.h then
					--love.graphics.setBackgroundColor(100,100*(i-1),100)
					--controller.setKeyBind(d[i])
					controller.updateKeys(c,d[i])
					--controller.setKeyBind(m.default[i])	
					-- ISSUE 1. I may have included this in the updateKeys function instead.  But perhaps "setKeyBind" is a better name for the function.
					-- iSSUE 2. controller.menu.default[i] is certainly incorrect now.
				end			
						
			end
			
			for i, v in ipairs(b.saveCustom) do
				if mx > v.x and mx < v.x + v.w and my > v.y and my < v.y + v.h then
					for j = 1,#k do
						c.custom[i][j] = k[j].bind
					end	
				end			
				
			end
			
			for i, v in ipairs(b.loadCustom) do
				if #c.custom[i] > 0 and
					mx > v.x and mx < v.x + v.w and my > v.y and my < v.y + v.h then
					for j = 1,#c.custom[i] do
						k[j].bind = c.custom[i][j]
						controller.updateKeys(c,c.custom[i]) -- ISSUE: confusion between the updateKeys() function and the setKeyBind() function
					end
				end	
			end
		end	
	--end
end

--=============================================
-- CALL BACK FUNCTIONS for demonstrating this object.
--=============================================
controller.demo = {}

function controller.demo.load() -- create controllers and their menus
	local numberOfControllers = 5

	local w = love.graphics.getWidth()
	local w = w/numberOfControllers
	local x = (w - controller.menu.w) / 2

	for i = 1, numberOfControllers do
		controller[i] = controller.newController()
		controller[i].menu.x = x+(i-1)*w
		controller.menu.load(controller[i])
	end
	
	controller.updateKeys(controller[2], controller[1].default[2])
	controller.updateKeys(controller[3], controller[1].default[3])
	controller.updateKeys(controller[4], {kp5,kp2,kp1,kp3}) -- this should only update some of the keys.
	controller.updateKeys(controller[5], {nil,nil,nil,nil,'1','2','3','4','5','6','7','8'}) -- this should only update some of the keys.

	
end

function controller.demo.draw()	-- draw controller menus
	for i, c in ipairs(controller) do
		controller.menu.draw(c)
	end
end

function controller.demo.keypressed(key)
	for i, c in ipairs(controller) do
		controller.menu.keypressed(c,key)
		controller.keypressed(c,key)
	end
end

function controller.demo.keyreleased(key)
	for i, c in ipairs(controller) do
		controller.keyreleased(c,key)
	end
end

function controller.demo.mousepressed(mx, my, button)
	for i, c in ipairs(controller) do
		controller.menu.mousepressed(c, mx, my, button)
	end
end
